<?php
namespace Teknisa\MAN\Services;

use Teknisa\MAN\Utilities\DateUtil;
use Teknisa\Libs\Util\Environment; 
use Teknisa\MAN\Factories\Ger97101FechamentoCupomFactory;


class Ger97101FechamentoCupomService {

    protected $environment;
    protected $fechamentoFactory;
    protected $obrigacoesService;

    const CUPOM_JA_APROVADO = "Redz_Ja_Aprov";
    const DATA_FORA_DO_PERIODO_PARAMETRIZADO = "DATA_FORA";
    const OPERACAO_COM_ERRO = "N";
    const OPERACAO_COM_SUCESSO = "S";
    const CANCELAMENTO_COM_SUCESSO = "9";
    const TIPO_RECEBIMENTO_FILIAL_ERROR = "R2";
    const MAPA_APURAC_ERROR = "Bloq";

    public function __construct(Environment $environment, Ger97101FechamentoCupomFactory $fechamentoFactory) {
        $this->environment = $environment;
        $this->fechamentoFactory = $fechamentoFactory;
    }

    /*
     * Função para realizar o procedimento de cancelamento de cupons aprovados.
     */
    public function aprovarCuponsFiscais($cupons, $valida = true) {
        // Inicializa array geral de erros nos cupons da requisição.
        
        $validationErrors = $valida?$this->preValidarCuponsFiscais($cupons):[];
        $cdoperador = $this->environment->getCdOperador();

        // Encerra o procedimento caso haja erros de pré-validação.
        if (!empty($validationErrors)) {
            return array('key' => 'preValidacaoErrors', 'data' => $validationErrors);
        }
        
        foreach ($cupons as $cupom) {
            
            // Cria uma data no formato DDMMYYYY, para verificar na tabela.
            $formattedDate = DateUtil::truncateString($cupom["DTREDUCAOZ"], DateUtil::FORMATO_BRASILEIRO_DATAHORA, 'dmY');
            // Cria uma chave contendo filial + caixa + data.
            $cupom["CDLISTAREDZ"] = $cupom["CDFILIAL"] . $cupom["CDCAIXA"] . $formattedDate;
            $params = array("NRORG" => $cupom["NRORG"]);
            $cupom["NRLISTAREDZ"] = $this->fechamentoFactory->getNextNrlistaredz($params);
            
            // Verifica se o cupom está pendente e se não está sendo aprovado
            // por outro usuário.
            if ($cupom["IDSTATUS"] == "P") {
                if ($this->cupomAlreadyBeingApproved($cupom)) {
                    $validationErrors[] = $this->getCupomWithErrorRow($cupom, self::CUPOM_JA_APROVADO);
                }
                else {
                    if (!$this->cupomIsBetweenParameterizedDates($cupom)) {
                        // Adiciona o cupom à lista de erros.
                        $validationErrors[] = $this->getCupomWithErrorRow($cupom, self::DATA_FORA_DO_PERIODO_PARAMETRIZADO);
                    }
                    else {
                        try {
                            // Seta o operador que está realizando a aprovação.
                            $cupom["CDOPERADOR"] = $cdoperador;

                            $this->setCupomBeingApproved($cupom);
                            
                            $procedureOutput = $this->callAprovacaoProcedure($cupom);
                            // Verifica se o cupom foi aprovado com sucesso.
                            if ($procedureOutput["P_RETORNO"] != self::OPERACAO_COM_SUCESSO || $procedureOutput["P_ERROS"] != null) {
                                $errors = $this->getErrorObjects($procedureOutput, $cupom);
                                
                                // Adiciona os erros à lista de erros.
                                foreach ($errors as $error) {
                                    // ALTERAÇÃO SOLICITADA PELO RAFAEL SOUZA - RETIRADA DO ERRO 'R1'
                                    if($error["ERRORCODE"] != 'R1')
                                        $validationErrors[] = $this->getCupomWithErrorRow($cupom, $error["ERRORCODE"], $error["ERRORPARAMS"]);
                                       
                                }
                            }
                            
                            $this->unsetCupomBeingApproved($cupom);
                        }
                        catch (\Exception $e) {
                            if ($this->cupomAlreadyBeingApproved($cupom)) {
                                $this->unsetCupomBeingApproved($cupom);
                            }
                            throw $e;
                        }
                    }
                }
            }
        }

        // Verifica se houve erros na procedure.
        if (!empty($validationErrors)) {
            // Retorna um array contendo os erros.
            return array('key' => 'procedureErrors', 'data' => $validationErrors);
        }
    }
    
    
    public function aprovarCuponsFiscaisPorLoja($cupons, $valida = true) {
        
        // Inicializa array geral de erros nos cupons da requisição.
        $validationErrors = $valida?$this->preValidarCuponsFiscaisPorLoja($cupons):[];
        $cdoperador = $this->environment->getCdOperador();

        // Encerra o procedimento caso haja erros de pré-validação.
        if (!empty($validationErrors)) {
            return array('key' => 'preValidacaoErrors', 'data' => $validationErrors);
        }
        
        foreach ($cupons as $cupom) {
            
            
            $IDCONFERIDO = 'S';
            $movlojadia = $this->fechamentoFactory->getMovLoja($cupom);
            
            foreach($movlojadia as $movloja){
                if($movloja['IDCONFERIDO'] != $IDCONFERIDO){
                    $IDCONFERIDO = 'N';
                }
            }
            
            // Cria uma data no formato DDMMYYYY, para verificar na tabela.
            $formattedDate = DateUtil::truncateString($cupom["DTREDUCAOZ"], DateUtil::FORMATO_BRASILEIRO_DATAHORA, 'dmY');
            // Cria uma chave contendo filial + loja + data.
            $cupom["CDLISTAREDZ"] = $cupom["CDFILIAL"] . $cupom["CDLOJA"] . $formattedDate;
            $params = array("NRORG" => $cupom["NRORG"]);
            $cupom["NRLISTAREDZ"] = $this->fechamentoFactory->getNextNrlistaredz($params);
            
            // Verifica se o cupom está pendente e se não está sendo aprovado
            // por outro usuário.
            if (($cupom["IDSTATUS"] == "P") && ($IDCONFERIDO == 'S')) {
                if ($this->cupomAlreadyBeingApprovedLoja($cupom)) {
                    $validationErrors[] = $this->getCupomWithErrorRowPorLoja($cupom, self::CUPOM_JA_APROVADO);
                }
                else {
                    if (!$this->cupomIsBetweenParameterizedDates($cupom)) {
                        // Adiciona o cupom à lista de erros.
                        $validationErrors[] = $this->getCupomWithErrorRowPorLoja($cupom, self::DATA_FORA_DO_PERIODO_PARAMETRIZADO);
                    }
                    else {
                        try {
                            // Seta o operador que está realizando a aprovação.
                            $cupom["CDOPERADOR"] = $cdoperador;
                            
                            $this->setCupomBeingApproved($cupom);
                            
                            $procedureOutput = $this->callAprovacaoProcedurePorLoja($cupom);
                            //var_dump($procedureOutput);die;
                            // Verifica se o cupom foi aprovado com sucesso.
                            if ($procedureOutput["P_RETORNO"] != self::OPERACAO_COM_SUCESSO || $procedureOutput["P_ERROS"] != null) {
                                $errors = $this->getErrorObjects($procedureOutput, $cupom);
                               // var_dump($errors);die;
                                // Adiciona os erros à lista de erros.
                                foreach ($errors as $error) {
                                    // ALTERAÇÃO SOLICITADA PELO RAFAEL SOUZA - RETIRADA DO ERRO 'R1'
                                    if($error["ERRORCODE"] != 'R1')
                                        $validationErrors[] = $this->getCupomWithErrorRowPorLoja($cupom, $error["ERRORCODE"], $error["ERRORPARAMS"]);
                                }
                            }
                            
                            $this->unsetCupomBeingApproved($cupom);
                        }
                        catch (\Exception $e) {
                            if ($this->cupomAlreadyBeingApprovedLoja($cupom)) {
                                $this->unsetCupomBeingApproved($cupom);
                            }
                            throw $e;
                        }
                    }
                }
            }  
        }
        // Verifica se houve erros na procedure.
        if (!empty($validationErrors)) {
            // Retorna um array contendo os erros.
            return array('key' => 'procedureErrors', 'data' => $validationErrors);
        }
    }

    /*
     * Função auxiliar para realizar a chamada da procedure de aprovação
     * de cupons.
     */
    private function callAprovacaoProcedure($cupom) {
        $sysdate = DateUtil::truncateString(DateUtil::getDataAtual(true), DateUtil::FORMATO_BRASILEIRO_DATAHORA, 'd/m/Y');
        
        $params = array(
            "CDFILIAL" => $cupom["CDFILIAL"],
            "CDCAIXA" => $cupom["CDCAIXA"],
            "DTREDUZ" => $cupom["DTREDUCAOZ"],
            "OPERADOR" => $cupom["CDOPERADOR"],
            "IDORIGEMREDZ" => $cupom["IDORIGEMREDZ"],
            "SYSDATE" => $sysdate,
        );
        return $this->fechamentoFactory->callAprovacaoProcedure($params);
    }
    
    private function callAprovacaoProcedurePorLoja($cupom) {
        $params = array(
            "CDFILIAL" => $cupom["CDFILIAL"],
            "CDLOJA" => $cupom["CDLOJA"],
            "DTREDUZ" => $cupom["DTREDUCAOZ"],
            "OPERADOR" => $cupom["CDOPERADOR"],
            "IDORIGEMREDZ" => $cupom["IDORIGEMREDZ"]
        );
        return $this->fechamentoFactory->callAprovacaoProcedurePorLoja($params);
    }

    /*
     * Função auxiliar para realizar a chamada da procedure de cancelamento
     * de cupons.
     */
    private function callCancelamentoProcedure($cupom) {
        $params = array(
            "CDFILIAL" => $cupom["CDFILIAL"],
            "CDCAIXA" => $cupom["CDCAIXA"],
            "DTREDUZ" => $cupom["DTREDUCAOZ"],
            "IDORIGEMREDZ" => $cupom["IDORIGEMREDZ"]
        );
        return $this->fechamentoFactory->callCancelamentoProcedure($params);
    }
    
    private function callCancelamentoProcedurePorLoja($cupom) {
        $params = array(
            "CDFILIAL" => $cupom["CDFILIAL"],
            "CDLOJA" => $cupom["CDLOJA"],
            "DTREDUZ" => $cupom["DTREDUCAOZ"],
            "IDORIGEMREDZ" => $cupom["IDORIGEMREDZ"]
        );
        return $this->fechamentoFactory->callCancelamentoProcedurePorLoja($params);
    }

    /*
     * Função para realizar o procedimento de cancelamento de cupons aprovados.
     */
    public function cancelarCuponsFiscais($cupons) {
        $validationErrors = array();
        
        foreach ($cupons as $cupom) {
            // Verifica se o cupom está aprovado.
            if (($cupom["IDSTATUS"] == "A") || ($cupom["IDSTATUS"] == "E")) {
                if (!$this->cupomIsBetweenParameterizedDates($cupom)) {
                    // Adiciona o cupom à lista de erros.
                    $validationErrors[] = $this->getCupomWithErrorRow($cupom, self::DATA_FORA_DO_PERIODO_PARAMETRIZADO);
                }
                else if (!$this->mapaApuracaoIsValid($cupom)) {
                    // Adiciona o cupom à lista de erros.
                    $validationErrors[] = $this->getCupomWithErrorRow($cupom, self::MAPA_APURAC_ERROR);
                }
                else {
                    $procedureOutput = $this->callCancelamentoProcedure($cupom);
                    // Verifica se o cupom foi cancelado com sucesso.
                    if ($procedureOutput["P_RETORNO"] != self::CANCELAMENTO_COM_SUCESSO) {
                        $errors = $this->getErrorObjects($procedureOutput, $cupom);
                        // Adiciona os erros à lista de erros.
                        foreach ($errors as $error) {
                            // ALTERAÇÃO SOLICITADA PELO RAFAEL SOUZA - RETIRADA DO ERRO 'R1'
                            if($error["ERRORCODE"] != 'R1')
                                $validationErrors[] = $this->getCupomWithErrorRow($cupom, $error["ERRORCODE"], $error["ERRORPARAMS"]);
                        }
                    }
                }
            }
        }

        // Verifica se há erros a ser exibidos.
        if (!empty($validationErrors)) {
            // Retorna o array de erros.
            return $validationErrors;
        }
    }

    public function cancelarCuponsFiscaisPorLoja($cupons) {
        $validationErrors = array();

        foreach ($cupons as $cupom) {
            // Verifica se o cupom está aprovado.
            if (($cupom["IDSTATUS"] == "A") || ($cupom["IDSTATUS"] == "E")) {
                if (!$this->cupomIsBetweenParameterizedDates($cupom)) {
                    // Adiciona o cupom à lista de erros.
                    $validationErrors[] = $this->getCupomWithErrorRowPorLoja($cupom, self::DATA_FORA_DO_PERIODO_PARAMETRIZADO);
                }
                else if (!$this->mapaApuracaoIsValidPorLoja($cupom)) {
                    // Adiciona o cupom à lista de erros.
                    $validationErrors[] = $this->getCupomWithErrorRowPorLoja($cupom, self::MAPA_APURAC_ERROR);
                }
                else {
                    $procedureOutput = $this->callCancelamentoProcedurePorLoja($cupom);
                    // Verifica se o cupom foi cancelado com sucesso.
                    if ($procedureOutput["P_RETORNO"] != self::CANCELAMENTO_COM_SUCESSO) {
                        $errors = $this->getErrorObjects($procedureOutput, $cupom);
                        // Adiciona os erros à lista de erros.
                        foreach ($errors as $error) {
                            // ALTERAÇÃO SOLICITADA PELO RAFAEL SOUZA - RETIRADA DO ERRO 'R1'
                            if($error["ERRORCODE"] != 'R1'){
                                $validationErrors[] = $this->getCupomWithErrorRowPorLoja($cupom, $error["ERRORCODE"], $error["ERRORPARAMS"]);
                            }
                        }
                    }
                }
            }
        }

        // Verifica se há erros a ser exibidos.
        if (!empty($validationErrors)) {
            // Retorna o array de erros.
            return $validationErrors;
        }
    }
    
    /*
     * Função para realizar o procedimento de liberação de cupons para aprovação.
     */
    public function cancelarLiberacaoCuponsFiscais($cupons) {
        // Chama a função auxiliar para realizar a alteração.
        $this->updateCupomLiberacao($cupons, false);
    }

    /*
     * Função para realizar o procedimento de liberação de cupons para aprovação.
     */
    public function conferenciaCuponsFiscais($cupons, $option) {
        // Inicializa o array que conterá as informações a ser relatadas.
        $reportData = array(
            "fiscal" => array(),
            "lancamentos" => array(),
            "recebimentos" => array()
        );
        foreach ($cupons as $cupom) {
            $params = array(
                "NRORG" => $cupom["NRORG"],
                "CDFILIAL" => $cupom["CDFILIAL"],
                "CDCAIXA" => $cupom["CDCAIXA"],
                "DTREDUCAOZ" => $cupom["DTREDUCAOZ"]
            );
            if ($cupom["IDSTATUS"] == "A") {
                // Adiciona as informações de fiscal, lançamentos contábeis ou
                // recebimentos do cupom aprovado.
                if ($option == "fiscal") {
                    $reportData["fiscal"] = array_merge($reportData["fiscal"], $this->fechamentoFactory->getConferenciaFiscal($params));
                }
                else if ($option == "lancamentos") {
                    $reportData["lancamentos"] = array_merge($reportData["lancamentos"], $this->fechamentoFactory->getConferenciaLancamentosContabeis($params));
                }
                else if ($option == "recebimentos") {
                    $reportData["recebimentos"] = array_merge($reportData["recebimentos"], $this->fechamentoFactory->getConferenciaRecebimentos($params));
                }
            }
        }

        // Limpa os arrays que não têm dados.
        $reportData = array_filter($reportData);

        return $reportData;
    }
    
    public function conferenciaCuponsFiscaisPorLoja($cupons, $option) {
        // Inicializa o array que conterá as informações a ser relatadas.
        $reportData = array(
            "fiscalPorLoja" => array(),
            "lancamentosPorLoja" => array(),
            "recebimentosPorLoja" => array()
        );
        foreach ($cupons as $cupom) {
            $params = array(
                "NRORG" => $cupom["NRORG"],
                "CDFILIAL" => $cupom["CDFILIAL"],
                "CDLOJA" => $cupom["CDLOJA"],
                "DTREDUCAOZ" => $cupom["DTREDUCAOZ"]
            );
            if ($cupom["IDSTATUS"] == "A") {
                // Adiciona as informações de fiscal, lançamentos contábeis ou
                // recebimentos do cupom aprovado.
                if ($option == "fiscalPorLoja") {
                    $reportData["fiscalPorLoja"] = array_merge($reportData["fiscalPorLoja"], $this->fechamentoFactory->getConferenciaFiscalPorLoja($params));
                }
                else if ($option == "lancamentosPorLoja") {
                    $reportData["lancamentosPorLoja"] = array_merge($reportData["lancamentosPorLoja"], $this->fechamentoFactory->getConferenciaLancamentosContabeisPorLoja($params));
                }
                else if ($option == "recebimentosPorLoja") {
                    $reportData["recebimentosPorLoja"] = array_merge($reportData["recebimentosPorLoja"], $this->fechamentoFactory->getConferenciaRecebimentosPorLoja($params));
                }
            }
        }

        // Limpa os arrays que não têm dados.
        $reportData = array_filter($reportData);

        return $reportData;
    }

    /*
     * Função para verificar se outro usuário no sistema Delphi já está tentando
     * aprovar um determinado cupom.
     */
    public function cupomAlreadyBeingApproved($cupom) {
        
        $params = array(
            "CDLISTAREDZ" => $cupom["CDLISTAREDZ"],
            "NRORG" => $cupom["NRORG"]
        );
        // Retorna um cupom, caso ele esteja sendo validado por outro usuário.
        $cupomBeingValidated = $this->fechamentoFactory->getCupomListaReducaoZ($params);

        // Retorna true caso o cupom já esteja na tabela de cupons sendo validados.
        
        return (bool) $cupomBeingValidated;
    }
    
    /*
     * Função para verificar se outro usuário no sistema Delphi já está tentando
     * aprovar um determinado cupom.
     */
    public function cupomAlreadyBeingApprovedLoja($cupom) {
        // Cria uma data no formato DDMMYYYY, para verificar na tabela.
        $formattedDate = DateUtil::truncateString($cupom["DTREDUCAOZ"], DateUtil::FORMATO_BRASILEIRO_DATAHORA, 'dmY');
        // Cria uma chave contendo filial + loja + data.
        $cupom["CDLISTAREDZ"] = $cupom["CDFILIAL"] . $cupom["CDLOJA"] . $formattedDate;
        
        $params = array(
            "CDLISTAREDZ" => $cupom["CDLISTAREDZ"],
            "NRORG" => $cupom["NRORG"]
        );
        // Retorna um cupom, caso ele esteja sendo validado por outro usuário.
        $cupomBeingValidated = $this->fechamentoFactory->getCupomListaReducaoZ($params);

        // Retorna true caso o cupom já esteja na tabela de cupons sendo validados.
        
        return (bool) $cupomBeingValidated;
    }

    /*
     * Função para verificar se um cupom está dentro das datas parametrizadas
     * na tabela CONTOPECTBL.
     */
    private function cupomIsBetweenParameterizedDates($cupom) {
        // Cria uma data no formato DD/MM/YYYY.
        $formattedDate = DateUtil::truncateString($cupom["DTREDUCAOZ"]);

        $params = array(
            "CDFILIAL" => $cupom["CDFILIAL"],
            "DTREDUCAOZ" => $cupom["DTREDUCAOZ"],
            "NRORG" => $cupom["NRORG"]
        );
        // Retorna o código da filial, caso o cupom esteja dentro de uma data válida.
        $filialRow = $this->fechamentoFactory->getFilialBetweenParameterizedDates($params);

        // Retorna true caso o cupom já esteja na tabela de cupons sendo validados.
        return (bool) $filialRow;
    }

    /*
     * Função auxiliar para retornar um array com os dados de um cupom e um
     * código de erro.
     */
    private function getCupomWithErrorRow($cupom, $errorCode, $errorParams = null) {
        return array(
            "CDFILIAL" => $cupom["CDFILIAL"],
            "NMFILIAL" => $cupom["NMFILIAL"],
            "CDCAIXA" => $cupom["CDCAIXA"],
            "NMCAIXA" => $cupom["NMCAIXA"],
            "DTREDUCAOZ" => $cupom["DTREDUCAOZ"],
            "DESCFILIAL" => $cupom["CDFILIAL"] . " - " . $cupom["NMFILIAL"],
            "DESCCAIXA" => $cupom["CDCAIXA"] . " - " . $cupom["NMCAIXA"],
            "ERRORCODE" => $errorCode,
            "ERRORPARAMS" => $errorParams
        );
    }
    
    private function getCupomWithErrorRowPorLoja($cupom, $errorCode, $errorParams = null) {
        return array(
            "CDFILIAL" => $cupom["CDFILIAL"],
            "NMFILIAL" => $cupom["NMFILIAL"],
            "CDLOJA" => $cupom["CDLOJA"],
            "NMLOJA" => $cupom["NMLOJA"],
            "DTREDUCAOZ" => $cupom["DTREDUCAOZ"],
            "DESCFILIAL" => $cupom["CDFILIAL"] . " - " . $cupom["NMFILIAL"],
            "DESCLOJA" => $cupom["CDLOJA"] . " - " . $cupom["NMLOJA"],
            "ERRORCODE" => $errorCode,
            "ERRORPARAMS" => $errorParams
        );
    }
    
    /*
     * Função auxiliar para retornar um array contendo os erros retornados
     * pela chamada à procedure.
     */
    private function getErrorObjects($procedureOutput, $cupom) {
        $errors = array();
       
        if (!empty($procedureOutput["P_RETORNO"])) {
            $errorCodes = explode('_', $procedureOutput["P_RETORNO"]);
            foreach ($errorCodes as $errorCode) {
                switch ($errorCode) {
                    case 'R2':
                        $errorParams = "Alguns tipos de recebimento por filial não possuem tipo de conta ou cliente cadastrados";
                        break;
                    case 'R4':
                        $errorParams = "Ocorreram erros na geração dos impostos dos itens da redução Z.";
                        break;
                    case 'R6':
                        $errorParams = "Tipo de operação não informado.";
                        break;
                    case 'R7':
                        $errorParams = 'Erro na operação. Não há série de documentos fiscais de saída vinculada a(s) filial(is) da(s) redução(s) Z, selecionada(s), na tela de "Parâmetros de Gestão de Vendas"';
                        break;
                    case 'R8':
                        $errorParams = "Erro na operação. Não há modelo de documento fiscal cadastrado para o tipo de operação atual.";
                        break;
                    case 'R9':
                        $errorParams = "Erro na operação. Não há Cliente padrão para vendas cadastrado para a filial da Redução Z.";
                        break;
                    default:
                        $errorParams = "Ocorreu um erro ao realizar a operação. Codigo do erro :".$errorCode;
                } 
                
                // Ignora o erro "operação com erro" e "operação realizada com
                // sucesso", não os adicionando na lista de erros.
                if ($errorCode != self::OPERACAO_COM_ERRO && $errorCode != self::OPERACAO_COM_SUCESSO && $errorCode != self::CANCELAMENTO_COM_SUCESSO) {
                    $errorParams = null;
                    if ($errorCode == self::TIPO_RECEBIMENTO_FILIAL_ERROR) {
                        $errorParams = $this->getParsedTipoRecebimento($procedureOutput["P_TIPORECE"]);
                    }
                    // ALTERAÇÃO SOLICITADA PELO RAFAEL SOUZA - RETIRADA DO ERRO 'R1'
                    if($errorCode != 'R1')
                        $errors[] = array("ERRORCODE" => $errorCode, "ERRORPARAMS" => $errorParams);
                }
            }
        }
        
        if (!empty($procedureOutput["P_ERROS"])) {
            $errorCodes = explode('_', $procedureOutput["P_ERROS"]);
            foreach ($errorCodes as $errorCode) {
                // Ignora o erro "operação com erro" e "operação realizada com
                // sucesso", não os adicionando na lista de erros.
                if ($errorCode != self::OPERACAO_COM_ERRO && $errorCode != self::OPERACAO_COM_SUCESSO && $errorCode != self::CANCELAMENTO_COM_SUCESSO) {
                    $errorParams = null;
                    if ($errorCode == self::TIPO_RECEBIMENTO_FILIAL_ERROR) {
                        $errorParams = $this->getParsedTipoRecebimento($procedureOutput["P_TIPORECE"]);
                    }
                    // ALTERAÇÃO SOLICITADA PELO RAFAEL SOUZA - RETIRADA DO ERRO 'R1'
                    if($errorCode != 'R1'){
                        
                        if(isset($cupom["CDCAIXA"])){
                            $params = array(
                                "P_CDFILIAL"     => $cupom["CDFILIAL"],
                                "P_CDCAIXA"      => $cupom["CDCAIXA"],
                                "P_DATAINI"   => $cupom["DTREDUCAOZ"],
                                "P_CDERRO" => $cupom["IDORIGEMREDZ"]
                            );
                            $errorParams = $this->fechamentoFactory->getErrorDesc($params);
                             
                            $errors[] = array("ERRORCODE" => $errorCode, "ERRORPARAMS" => $errorParams);
                        }else{
                           $params = array(
                                "P_CDFILIAL"     => $cupom["CDFILIAL"],
                                "P_CDLOJA"      => $cupom["CDLOJA"],
                                "P_DATAINI"   => $cupom["DTREDUCAOZ"],
                                "P_CDERRO" => $cupom["IDORIGEMREDZ"]
                            );
                           // var_dump($params);die;
                            $errorParams = $this->fechamentoFactory->getErrorDescporLoja($params);
                            $errors[] = array("ERRORCODE" => $errorCode, "ERRORPARAMS" => $errorParams); 
                        }
                        
                    }
                }
            }
        }

        return $errors;
    }

    /*
     * Função auxiliar para retornar uma String
     */
    private function getParsedTipoRecebimento($tipoRecebimentoOutput) {
        $nrorg = $this->environment->getNrorg();
        $strTipoRece = "";
        $tipoReceCodes = explode('_', $tipoRecebimentoOutput);
        foreach ($tipoReceCodes as $cdtiporece) {
            $params = array(
                "NRORG" => $nrorg,
                "CDTIPORECE" => $cdtiporece
            );
            if ($strTipoRece != "") {
                $strTipoRece .= ",";
            }
            $strTipoRece .= " ".$cdtiporece." - ".$this->fechamentoFactory->getTipoReceName($params);
        }
        return $strTipoRece;
    }

    /*
     * Função auxiliar para retornar um array com os erros de pré-validação de
     * um cupom, caso existam. Retorna null ou um array vazio caso não existam.
     */
    private function getPreValidacaoErrors($cupom) {
        $cupomErrors = null;
        
        // Verifica se o cupom está pendente.
        if ($cupom["IDSTATUS"] == "P") {
            $params = array(
                "CDFILIAL"     => $cupom["CDFILIAL"],
                "CDCAIXA"      => $cupom["CDCAIXA"],
                "DTREDUCAOZ"   => $cupom["DTREDUCAOZ"],
                "IDORIGEMREDZ" => $cupom["IDORIGEMREDZ"],
                "FILIALVALIDA" => isset($cupom["FILIALVALIDA"]) ? $cupom["FILIALVALIDA"] : 'S'
            );
            
            $this->fechamentoFactory->callPreValidacaoProcedure($params);
            
            $params = array(
                "CDFILIAL"   => $cupom["CDFILIAL"],
                "CDCAIXA"    => $cupom["CDCAIXA"],
                "DTREDUCAOZ" => $cupom["DTREDUCAOZ"],
                "NRORG"      => $cupom["NRORG"]
            );
            
            $cupomErrors = $this->fechamentoFactory->getPreValidacaoErrors($params);
        }

        return $cupomErrors;
    }

    /*
     * Função para validar o mapa de apuração, dado um cupom.
     */
    private function mapaApuracaoIsValid($cupom) {
        $cupom["DTREDUCAOZ"] = DateUtil::truncateString($cupom["DTREDUCAOZ"]);
        $params = array(
            "IDORIGEMREDZ" => $cupom["IDORIGEMREDZ"],
            "DTREDUCAOZ" => $cupom["DTREDUCAOZ"],
            "CDCAIXA" => $cupom["CDCAIXA"],
            "CDFILIAL" => $cupom["CDFILIAL"],
            "NRORG" => $cupom["NRORG"]
        );

        // Retorna uma filial, caso o mapa não esteja válido.
        $cdfilial = $this->fechamentoFactory->mapaApuracaoIsValid($params);

        // Retorna true caso o cupom não pertença ao mapa de apuração.
        return !(bool) $cdfilial;
    }
    
    private function mapaApuracaoIsValidPorLoja($cupom) {
        $cupom["DTREDUCAOZ"] = DateUtil::truncateString($cupom["DTREDUCAOZ"]);
        $params = array(
            "IDORIGEMREDZ" => $cupom["IDORIGEMREDZ"],
            "DTREDUCAOZ" => $cupom["DTREDUCAOZ"],
            "CDLOJA" => $cupom["CDLOJA"],
            "CDFILIAL" => $cupom["CDFILIAL"],
            "NRORG" => $cupom["NRORG"]
        );

        // Retorna uma filial, caso o mapa não esteja válido.
        $cdfilial = $this->fechamentoFactory->mapaApuracaoIsValidPorLoja($params);

        // Retorna true caso o cupom não pertença ao mapa de apuração.
        return !(bool) $cdfilial;
    }

    /*
     * Função para realizar o procedimento de liberação de cupons para aprovação.
     */
    public function liberarAprovacaoCuponsFiscais($cupons) {
        // Chama a função auxiliar para realizar a alteração.
        $this->updateCupomLiberacao($cupons, true);
    }

    /*
     * Função para realizar o procedimento de pré-validação de cupons.
     * Retorna um array com os erros encontrados
     */
    public function preValidarCuponsFiscais($cupons) {
        // Inicializa array geral de erros nos cupons da requisição.
        $validationErrors = array();

        foreach ($cupons as $cupom) {
            // Chama a função auxiliar para validação e retorno de erros.
            $cupomErrors = $this->getPreValidacaoErrors($cupom);

            // Verifica se houve erros no cupom atual, adicionando-os ao
            // array geral.
            if (!empty($cupomErrors)) {
                $validationErrors = array_merge($validationErrors, $cupomErrors);
            }
        }

        return $validationErrors;
    }
    
    public function preValidarCuponsFiscaisPorLoja($cupons) {
        // Inicializa array geral de erros nos cupons da requisição.
        $validationErrors = array();
        
        
        foreach ($cupons as $loja) {
            
            $caixas = $this->fechamentoFactory->getCaixaPorLoja($loja);
            
            foreach ($caixas as $cupom) {
                // Chama a função auxiliar para validação e retorno de erros.
                $loja['CDCAIXA'] = $cupom['CDCAIXA'];
            
                $cupomErrors = $this->getPreValidacaoErrors($loja);
    
                // Verifica se houve erros no cupom atual, adicionando-os ao
                // array geral.
                if (!empty($cupomErrors)) {
                    $validationErrors = array_merge($validationErrors, $cupomErrors);
                }
            }
        }

        return $validationErrors;
    }

    /*
     * Função auxiliar para adicionar dados na tabela, indicando que um cupom
     * já está sendo aprovado.
     */
    private function setCupomBeingApproved($cupom) {
        $params = array(
            "CDLISTAREDZ" => $cupom["CDLISTAREDZ"],
            "NRLISTAREDZ" => $cupom["NRLISTAREDZ"],
            "NRORG" => $cupom["NRORG"]
        );
        $this->fechamentoFactory->insertCupomListaredz($params);
    }

    /*
     * Função auxiliar para remover dados na tabela, eliminando dados temporários.
     */
    private function unsetCupomBeingApproved($cupom) {
        $params = array(
            "CDLISTAREDZ" => $cupom["CDLISTAREDZ"],
            "NRLISTAREDZ" => $cupom["NRLISTAREDZ"],
            "NRORG" => $cupom["NRORG"]
        );
        $this->fechamentoFactory->deleteCupomListaredz($params);
    }

    /*
     * Função auxiliar para atualizar os estados dos cupons ao liberar
     * ou cancelar liberação para aprovação.
     */
    private function updateCupomLiberacao($cupons, $isToRelease) {
        $nrorg = $this->environment->getNrorg();
        $cdoperador = $this->environment->getCdOperador();
        $params = array(
            "NRORG" => $nrorg
        );
        // Verifica se é para liberar ou cancelar a liberação.
        $liberaprov = $isToRelease ? 'S' : 'N';
        $parametrosFiliais = $this->fechamentoFactory->getParametroLiberacaoFilial($params);

        foreach ($cupons as $cupom) {
            // Verifica se o cupom está pendente, não liberado e a filial está parametrizada.
            if ($cupom["IDSTATUS"] == "P" && $cupom["IDLIBERAPROV"] != $liberaprov && $parametrosFiliais[$cupom["CDFILIAL"]] == 'S') {
                // Verifica se o cupom é de venda.
                if ($cupom["IDORIGEMREDZ"] == "NFCE" || $cupom["IDORIGEMREDZ"] == "SAT") {
                    $params = array(
                        "IDLIBERAPROV" => $liberaprov,
                        "CDOPERLIBERCUPOM" => $isToRelease ? $cdoperador : null,
                        "CDFILIAL" => $cupom["CDFILIAL"],
                        "CDCAIXA" => $cupom["CDCAIXA"],
                        "IDORIGEMREDZ" => $cupom["IDORIGEMREDZ"],
                        "DTENTRVENDA" => $cupom["DTREDUCAOZ"],
                        "NRORG" => $nrorg
                    );
                    $this->fechamentoFactory->updateVenda($params);
                }
                else {
                    $params = array(
                        "IDLIBERAPROV" => $liberaprov,
                        "CDOPERLIBERCUPOM" => $isToRelease ? $cdoperador : null,
                        "CDFILIAL" => $cupom["CDFILIAL"],
                        "CDCAIXA" => $cupom["CDCAIXA"],
                        "DTREDUCAOZ" => $cupom["DTREDUCAOZ"],
                        "NRORG" => $nrorg
                    );
                    $this->fechamentoFactory->updateReducaoZ($params);
                }
            }
        }
    }
    
    
    
    public function limparbloqueioporLoja($row){
         foreach ($row as $loja) {
            
            $caixas = $this->fechamentoFactory->getCaixaPorLoja($loja);
            
            foreach ($caixas as $cupom) {
                $loja['CDCAIXA'] = $cupom['CDCAIXA'];
                // Cria uma data no formato DDMMYYYY, para verificar na tabela.
                $formattedDate = DateUtil::truncateString($loja["DTREDUCAOZ"], DateUtil::FORMATO_BRASILEIRO_DATAHORA, 'dmY');
                // Cria uma chave contendo filial + loja + data.
                 $loja["CDLISTAREDZ"] = $loja["CDFILIAL"] . $loja["CDLOJA"] . $formattedDate;
                 if($loja['IDORIGEMREDZ'] == 'NFCE' || $loja['IDORIGEMREDZ'] == 'SAT'){
                     $updateLojaNFCESAT = $this->fechamentoFactory->updateLojaNFCESAT($loja);
                 }else{
                     $updateLoja = $this->fechamentoFactory->updateLoja($loja);
                 }
                
                
                $deletebloqueio = $this->fechamentoFactory->deletebloqueio($loja);
            }
        }
    }
    
    public function limparbloqueio($row){
            foreach ($row as $caixa) {
               
                // Cria uma data no formato DDMMYYYY, para verificar na tabela.
                $formattedDate = DateUtil::truncateString($caixa["DTREDUCAOZ"], DateUtil::FORMATO_BRASILEIRO_DATAHORA, 'dmY');
                // Cria uma chave contendo filial + loja + data.
                $caixa["CDLISTAREDZ"] = $caixa["CDFILIAL"] . $caixa["CDCAIXA"] . $formattedDate;
              
                if($caixa['IDORIGEMREDZ'] == 'NFCE' || $caixa['IDORIGEMREDZ'] == 'SAT'){
                    $updateCaixaNFCESAT = $this->fechamentoFactory->updateCaixaNFCESAT($caixa);
                }else{
                    $updateCaixa = $this->fechamentoFactory->updateCaixa($caixa);
                }
                
                $deletebloqueio = $this->fechamentoFactory->deletebloqueio($caixa);
            }
        
    }
}